@@ -20,6 +20,7 @@
 
 #include "akodelib.h"
 #include "sink.h"
+#include "auto_sink.h"
 
 namespace aKode {
 
@@ -31,11 +32,21 @@ SinkPluginHandler::SinkPluginHandler(const string lib) : sink_plugin(0)
 
 bool SinkPluginHandler::load(const string name)
 {
+    if (library_loaded) return false;
+
     bool res = PluginHandler::load(name+"_sink");
     if (res)
         sink_plugin = (SinkPlugin*)loadPlugin(name+"_sink");
+    else
+        if (name == "auto")
+            sink_plugin = &auto_sink;
+        else
+            return false; // plugin not found
 
-    return res && sink_plugin;
+    if (!res)
+        unload();
+
+    return res;
 }
 
 Sink* SinkPluginHandler::openSink() {
