@@ -12,6 +12,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <kurl.h>
+
 #include <config.h>
 #include <qobjectlist.h>
 
@@ -25,9 +27,11 @@
 #include "searchplaylist.h"
 #include "folderplaylist.h"
 #include "historyplaylist.h"
+#include "upcomingplaylist.h"
 #include "directorylist.h"
 #include "mediafiles.h"
 #include "playermanager.h"
+#include "tracksequencemanager.h"
 
 #include <kiconloader.h>
 #include <kactionclasses.h>
@@ -37,6 +41,7 @@
 #include <kfiledialog.h>
 
 #include <qwidgetstack.h>
+#include <qhbox.h>
 
 #define widget (kapp->mainWidget())
 
@@ -49,12 +54,23 @@ using namespace ActionCollection;
 PlaylistCollection::PlaylistCollection(QWidgetStack *playlistStack) :
     m_playlistStack(playlistStack),
     m_historyPlaylist(0),
+    m_upcomingPlaylist(0),
     m_importPlaylists(true),
     m_searchEnabled(true),
-    m_playing(false)
+    m_playing(false),
+    m_showMorePlaylist(0),
+    m_belowShowMorePlaylist(0),
+    m_dynamicPlaylist(0),
+    m_belowDistraction(0),
+    m_distraction(0)
 {
     m_actionHandler = new ActionHandler(this);
     PlayerManager::instance()->setPlaylistInterface(this);
+
+    // KDirLister's auto error handling seems to crash JuK during startup in
+    // readConfig().
+
+    m_dirLister.setAutoErrorHandlingEnabled(false, playlistStack);
     readConfig();
 }
 
@@ -92,6 +108,13 @@ void PlaylistCollection::playFirst()
     currentChanged();
 }
 
+void PlaylistCollection::playNextAlbum()
+{
+    m_playing = true;
+    currentPlaylist()->playNextAlbum();
+    currentChanged();
+}
+
 void PlaylistCollection::playPrevious()
 {
     m_playing = true;
@@ -110,7 +133,7 @@ void PlaylistCollection::stop()
 {
     m_playing = false;
     currentPlaylist()->stop();
-    currentChanged();
+    dataChanged();
 }
 
 bool PlaylistCollection::playing() const
@@ -138,6 +161,60 @@ void PlaylistCollection::createPlaylist(const QString &name)
     raise(new Playlist(this, name));
 }
 
+void PlaylistCollection::createDynamicPlaylist(const PlaylistList &playlists)
+{
+    if(m_dynamicPlaylist)
+        m_dynamicPlaylist->setPlaylists(playlists);
+    else
+        m_dynamicPlaylist =
+            new DynamicPlaylist(playlists, this, i18n("Dynamic List"), "midi", false, true);
+
+    PlaylistCollection::raise(m_dynamicPlaylist);
+}
+
+void PlaylistCollection::showMore(const QString &artist, const QString &album)
+{
+
+    PlaylistList playlists;
+    PlaylistSearch::ComponentList components;
+
+    if(currentPlaylist() != CollectionList::instance() &&
+       currentPlaylist() != m_showMorePlaylist)
+    {
+        playlists.append(currentPlaylist());
+    }
+
+    playlists.append(CollectionList::instance());
+
+    { // Just setting off the artist stuff in its own block.
+        ColumnList columns;
+        columns.append(PlaylistItem::ArtistColumn);
+        PlaylistSearch::Component c(artist, false, columns,
+                                    PlaylistSearch::Component::Exact);
+        components.append(c);
+    }
+
+    if(!album.isNull()) {
+        ColumnList columns;
+        columns.append(PlaylistItem::AlbumColumn);
+        PlaylistSearch::Component c(album, false, columns,
+                                    PlaylistSearch::Component::Exact);
+        components.append(c);
+    }
+
+    PlaylistSearch search(playlists, components, PlaylistSearch::MatchAll);
+
+    if(m_showMorePlaylist)
+        m_showMorePlaylist->setPlaylistSearch(search);
+    else
+        m_showMorePlaylist = new SearchPlaylist(this, search, i18n("Now Playing"), false, true);
+
+    m_belowShowMorePlaylist = visiblePlaylist();
+
+    PlaylistCollection::setupPlaylist(m_showMorePlaylist, QString::null);
+    PlaylistCollection::raise(m_showMorePlaylist);
+}
+
 void PlaylistCollection::removeTrack(const QString &playlist, const QStringList &files)
 {
     Playlist *p = playlistByName(playlist);
@@ -161,9 +238,12 @@ void PlaylistCollection::removeTrack(const QString &playlist, const QStringList
 
 QString PlaylistCollection::playlist() const
 {
-    if(currentPlaylist())
-        return currentPlaylist()->name();
-    return QString::null;
+    return visiblePlaylist() ? visiblePlaylist()->name() : QString::null;
+}
+
+QString PlaylistCollection::playingPlaylist() const
+{
+    return currentPlaylist() && m_playing ? currentPlaylist()->name() : QString::null;
 }
 
 void PlaylistCollection::setPlaylist(const QString &playlist)
@@ -200,7 +280,12 @@ void PlaylistCollection::open(const QStringList &l)
     if(files.isEmpty())
         return;
 
-    if(visiblePlaylist() == CollectionList::instance() ||
+    bool justPlaylists = true;
+
+    for(QStringList::ConstIterator it = l.begin(); it != l.end() && justPlaylists; ++it)
+        justPlaylists = !MediaFiles::isPlaylistFile(*it);
+
+    if(visiblePlaylist() == CollectionList::instance() || justPlaylists ||
        KMessageBox::questionYesNo(
            widget,
            i18n("Do you want to add these items to the current list or to the collection list?"),
@@ -208,10 +293,10 @@ void PlaylistCollection::open(const QStringList &l)
            KGuiItem(i18n("Current")),
            KGuiItem(i18n("Collection"))) == KMessageBox::No)
     {
-        CollectionList::instance()->addFiles(files, m_importPlaylists);
+        CollectionList::instance()->addFiles(files);
     }
     else
-        visiblePlaylist()->addFiles(files, m_importPlaylists);
+        visiblePlaylist()->addFiles(files);
 
     dataChanged();
 }
@@ -221,36 +306,44 @@ void PlaylistCollection::open(const QString &playlist, const QStringList &files)
     Playlist *p = playlistByName(playlist);
 
     if(p)
-        p->addFiles(files, m_importPlaylists);
+        p->addFiles(files);
 }
 
 void PlaylistCollection::addFolder()
 {
+    kdDebug(65432) << k_funcinfo << endl;
     DirectoryList l(m_folderList, m_importPlaylists, widget, "directoryList");
     DirectoryList::Result result = l.exec();
 
     if(result.status == QDialog::Accepted) {
 
+        m_dirLister.blockSignals(true);
+
+        const bool reload = m_importPlaylists != result.addPlaylists;
+        m_importPlaylists = result.addPlaylists;
+
         for(QStringList::Iterator it = result.addedDirs.begin();
             it != result.addedDirs.end(); it++)
         {
-            m_dirWatch.addDir(*it, false, true);
-	    m_folderList.append(*it);
+            m_dirLister.openURL(*it, true);
+            m_folderList.append(*it);
         }
 
         for(QStringList::Iterator it = result.removedDirs.begin();
             it !=  result.removedDirs.end(); it++)
         {
-            m_dirWatch.removeDir(*it);
+            m_dirLister.stop(*it);
             m_folderList.remove(*it);
         }
 
-        m_importPlaylists = result.addPlaylists;
-
-        if(result.addPlaylists && !m_importPlaylists)
+        if(reload)
             open(m_folderList);
         else if(!result.addedDirs.isEmpty())
             open(result.addedDirs);
+
+        saveConfig();
+
+        m_dirLister.blockSignals(false);
     }
 }
 
@@ -285,15 +378,10 @@ void PlaylistCollection::saveAs()
     visiblePlaylist()->saveAs();
 }
 
-void PlaylistCollection::remove()
-{
-    // implemented in subclass
-}
-
 void PlaylistCollection::reload()
 {
     if(visiblePlaylist() == CollectionList::instance())
-        CollectionList::instance()->addFiles(m_folderList, m_importPlaylists);
+        CollectionList::instance()->addFiles(m_folderList);
     else
         visiblePlaylist()->slotReload();
 
@@ -315,11 +403,6 @@ void PlaylistCollection::editSearch()
     }
 }
 
-void PlaylistCollection::setCanDeletePlaylist(bool canDelete)
-{
-    // Implemented in subclass
-}
-
 void PlaylistCollection::removeItems()
 {
     visiblePlaylist()->slotRemoveSelectedItems();
@@ -335,6 +418,23 @@ void PlaylistCollection::renameItems()
     visiblePlaylist()->slotRenameFile();
 }
 
+void PlaylistCollection::addCovers(bool fromFile)
+{
+    visiblePlaylist()->slotAddCover(fromFile);
+    dataChanged();
+}
+
+void PlaylistCollection::removeCovers()
+{
+    visiblePlaylist()->slotRemoveCover();
+    dataChanged();
+}
+
+void PlaylistCollection::viewCovers()
+{
+    visiblePlaylist()->slotViewCover();
+}
+
 PlaylistItemList PlaylistCollection::selectedItems()
 {
     return visiblePlaylist()->selectedItems();
@@ -342,7 +442,7 @@ PlaylistItemList PlaylistCollection::selectedItems()
 
 void PlaylistCollection::scanFolders()
 {
-    CollectionList::instance()->addFiles(m_folderList, m_importPlaylists);
+    CollectionList::instance()->addFiles(m_folderList);
 
     if(CollectionList::instance()->count() == 0)
         addFolder();
@@ -407,8 +507,6 @@ HistoryPlaylist *PlaylistCollection::historyPlaylist() const
 
 void PlaylistCollection::setHistoryPlaylistEnabled(bool enable)
 {
-//    kdDebug(65432) << k_funcinfo << enable << endl;
-
     if((enable && m_historyPlaylist) || (!enable && !m_historyPlaylist))
         return;
 
@@ -424,6 +522,36 @@ void PlaylistCollection::setHistoryPlaylistEnabled(bool enable)
     }
 }
 
+UpcomingPlaylist *PlaylistCollection::upcomingPlaylist() const
+{
+    return m_upcomingPlaylist;
+}
+
+void PlaylistCollection::setUpcomingPlaylistEnabled(bool enable)
+{
+    if((enable && m_upcomingPlaylist) || (!enable && !m_upcomingPlaylist))
+        return;
+
+    if(enable) {
+        action<KToggleAction>("showUpcoming")->setChecked(true);
+        if(!m_upcomingPlaylist)
+            m_upcomingPlaylist = new UpcomingPlaylist(this);
+
+        setupPlaylist(m_upcomingPlaylist, "today");
+    }
+    else {
+        action<KToggleAction>("showUpcoming")->setChecked(false);
+        bool raiseCollection = m_playlistStack->visibleWidget() == m_upcomingPlaylist;
+        delete m_upcomingPlaylist;
+        m_upcomingPlaylist = 0;
+
+        if(raiseCollection) {
+            kapp->processEvents(); // Seems to stop a crash, weird.
+            raise(CollectionList::instance());
+        }
+    }
+}
+
 QObject *PlaylistCollection::object() const
 {
     return m_actionHandler;
@@ -431,6 +559,12 @@ QObject *PlaylistCollection::object() const
 
 Playlist *PlaylistCollection::currentPlaylist() const
 {
+    if(m_belowDistraction)
+        return m_belowDistraction;
+
+    if(m_upcomingPlaylist && m_upcomingPlaylist->active())
+        return m_upcomingPlaylist;
+
     if(Playlist::playingItem())
         return Playlist::playingItem()->playlist();
     else
@@ -442,6 +576,47 @@ Playlist *PlaylistCollection::visiblePlaylist() const
     return static_cast<Playlist *>(m_playlistStack->visibleWidget());
 }
 
+void PlaylistCollection::raise(Playlist *playlist)
+{
+    if(m_showMorePlaylist && currentPlaylist() == m_showMorePlaylist)
+        m_showMorePlaylist->lower(playlist);
+    if(m_dynamicPlaylist && currentPlaylist() == m_dynamicPlaylist)
+        m_dynamicPlaylist->lower(playlist);
+
+    TrackSequenceManager::instance()->setCurrentPlaylist(playlist);
+    playlist->applySharedSettings();
+    playlist->setSearchEnabled(m_searchEnabled);
+    m_playlistStack->raiseWidget(playlist);
+    clearShowMore(false);
+    dataChanged();
+}
+
+void PlaylistCollection::raiseDistraction()
+{
+    if(m_belowDistraction)
+        return;
+
+    m_belowDistraction = currentPlaylist();
+
+    if(!m_distraction) {
+        m_distraction = new QHBox(m_playlistStack);
+        m_playlistStack->addWidget(m_distraction);
+    }
+
+    m_playlistStack->raiseWidget(m_distraction);
+}
+
+void PlaylistCollection::lowerDistraction()
+{
+    if(!m_distraction)
+        return;
+
+    if(m_belowDistraction)
+        m_playlistStack->raiseWidget(m_belowDistraction);
+
+    m_belowDistraction = 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // protected methods
 ////////////////////////////////////////////////////////////////////////////////
@@ -451,13 +626,6 @@ QWidgetStack *PlaylistCollection::playlistStack() const
     return m_playlistStack;
 }
 
-void PlaylistCollection::raise(Playlist *playlist)
-{
-    playlist->setSearchEnabled(m_searchEnabled);
-    m_playlistStack->raiseWidget(playlist);
-    dataChanged();
-}
-
 void PlaylistCollection::setupPlaylist(Playlist *playlist, const QString &)
 {
     if(!playlist->fileName().isNull())
@@ -480,12 +648,39 @@ bool PlaylistCollection::containsPlaylistFile(const QString &file) const
     return m_playlistFiles.contains(file);
 }
 
+bool PlaylistCollection::showMoreActive() const
+{
+    return visiblePlaylist() == m_showMorePlaylist;
+}
+
+void PlaylistCollection::clearShowMore(bool raisePlaylist)
+{
+    if(!m_showMorePlaylist)
+        return;
+
+    if(raisePlaylist) {
+        if(m_belowShowMorePlaylist)
+            raise(m_belowShowMorePlaylist);
+        else
+            raise(CollectionList::instance());
+    }
+
+    m_belowShowMorePlaylist = 0;
+}
+
 void PlaylistCollection::enableDirWatch(bool enable)
 {
-    if(enable)
-        m_dirWatch.startScan(false);
-    else
-        m_dirWatch.stopScan();
+    QObject *collection = CollectionList::instance();
+
+    m_dirLister.disconnect(object());
+    if(enable) {
+        QObject::connect(&m_dirLister, SIGNAL(newItems(const KFileItemList &)),
+                object(), SLOT(slotNewItems(const KFileItemList &)));
+        QObject::connect(&m_dirLister, SIGNAL(refreshItems(const KFileItemList &)),
+                collection, SLOT(slotRefreshItems(const KFileItemList &)));
+        QObject::connect(&m_dirLister, SIGNAL(deleteItem(KFileItem *)),
+                collection, SLOT(slotDeleteItem(KFileItem *)));
+    }
 }
 
 QString PlaylistCollection::playlistNameDialog(const QString &caption,
@@ -526,19 +721,29 @@ QString PlaylistCollection::uniquePlaylistName(const QString &suggest) const
     return s;
 }
 
-void PlaylistCollection::addName(const QString &name)
+void PlaylistCollection::addNameToDict(const QString &name)
 {
     m_playlistNames.insert(name);
 }
 
-void PlaylistCollection::removeName(const QString &name)
+void PlaylistCollection::addFileToDict(const QString &file)
+{
+    m_playlistFiles.insert(file);
+}
+
+void PlaylistCollection::removeNameFromDict(const QString &name)
 {
     m_playlistNames.remove(name);
 }
 
+void PlaylistCollection::removeFileFromDict(const QString &file)
+{
+    m_playlistFiles.remove(file);
+}
+
 void PlaylistCollection::dirChanged(const QString &path)
 {
-    CollectionList::instance()->addFiles(path, m_importPlaylists);
+    CollectionList::instance()->addFiles(path);
 }
 
 Playlist *PlaylistCollection::playlistByName(const QString &name) const
@@ -559,6 +764,10 @@ Playlist *PlaylistCollection::playlistByName(const QString &name) const
     return list;
 }
 
+void PlaylistCollection::newItems(const KFileItemList &list) const
+{
+    CollectionList::instance()->slotNewItems(list);
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 // private methods
@@ -568,25 +777,18 @@ void PlaylistCollection::readConfig()
 {
     KConfigGroup config(KGlobal::config(), "Playlists");
 
-    m_importPlaylists = config.readBoolEntry("ImportPlaylists", true);
-    m_folderList      = config.readPathListEntry("DirectoryList");
+    m_importPlaylists  = config.readBoolEntry("ImportPlaylists", true);
+    m_folderList       = config.readPathListEntry("DirectoryList");
 
-    QObject::connect(&m_dirWatch, SIGNAL(dirty(const QString &)),
-            object(), SLOT(slotDirChanged(const QString &)));
-
-    for(QStringList::ConstIterator it = m_folderList.begin();
-        it != m_folderList.end(); ++it)
-    {
-        m_dirWatch.addDir(*it, false, true);
-    }
-
-    m_dirWatch.startScan();
+    for(QStringList::ConstIterator it = m_folderList.begin(); it != m_folderList.end(); ++it)
+        m_dirLister.openURL(*it, true);
 }
 
 void PlaylistCollection::saveConfig()
 {
     KConfigGroup config(KGlobal::config(), "Playlists");
     config.writeEntry("ImportPlaylists", m_importPlaylists);
+    config.writeEntry("showUpcoming", action<KToggleAction>("showUpcoming")->isChecked());
     config.writePathEntry("DirectoryList", m_folderList);
 }
 
@@ -627,7 +829,9 @@ PlaylistCollection::ActionHandler::ActionHandler(PlaylistCollection *collection)
 #endif
 
 
-    createAction(i18n("Play First Track"),SLOT(slotPlayFirst()),    "playFirst");
+    createAction(i18n("Play First Track"),SLOT(slotPlayFirst()),     "playFirst");
+    createAction(i18n("Play Next Album"), SLOT(slotPlayNextAlbum()), "forwardAlbum", "next");
+
     createAction(i18n("Open..."),         SLOT(slotOpen()),         "file_open", "fileopen", "CTRL+o");
     createAction(i18n("Add &Folder..."),  SLOT(slotAddFolder()),    "openDirectory", "fileopen");
     createAction(i18n("&Rename..."),      SLOT(slotRename()),       "renamePlaylist", "lineedit");
@@ -642,12 +846,29 @@ PlaylistCollection::ActionHandler::ActionHandler(PlaylistCollection *collection)
     createAction(i18n("Refresh"),         SLOT(slotRefreshItems()), "refresh", "reload");
     createAction(i18n("&Rename File"),    SLOT(slotRenameItems()),  "renameFile", "filesaveas", "CTRL+r");
 
+    menu = new KActionMenu(i18n("Cover Manager"), QString::null, actions(), "coverManager");
+    menu->setIconSet(SmallIconSet("image"));
+    menu->insert(createAction(i18n("&View Cover"),
+        SLOT(slotViewCovers()), "viewCover", "viewmag"));
+    menu->insert(createAction(i18n("Get Cover From &File..."),
+        SLOT(slotAddLocalCover()), "addCover", "fileimport", "CTRL+SHIFT+f"));
+    menu->insert(createAction(i18n("Get Cover From &Internet..."), 
+        SLOT(slotAddInternetCover()), "googleCover", "connect_established", "CTRL+SHIFT+g"));
+    menu->insert(createAction(i18n("&Delete Cover"),
+        SLOT(slotRemoveCovers()), "removeCover", "editdelete"));
+
     KToggleAction *historyAction =
         new KToggleAction(i18n("Show &History"), "history",  0, actions(), "showHistory");
     historyAction->setCheckedState(i18n("Hide &History"));
 
+    KToggleAction *upcomingAction =
+        new KToggleAction(i18n("Show &Play Queue"), "today", 0, actions(), "showUpcoming");
+    upcomingAction->setCheckedState(i18n("Hide &Play Queue"));
+
     connect(action<KToggleAction>("showHistory"), SIGNAL(toggled(bool)),
             this, SLOT(slotSetHistoryPlaylistEnabled(bool)));
+    connect(action<KToggleAction>("showUpcoming"), SIGNAL(toggled(bool)),
+            this, SLOT(slotSetUpcomingPlaylistEnabled(bool)));
 }
 
 KAction *PlaylistCollection::ActionHandler::createAction(const QString &text,
@@ -662,5 +883,7 @@ KAction *PlaylistCollection::ActionHandler::createAction(const QString &text,
         return new KAction(text, icon, shortcut, this, slot, actions(), name);
 }
 
-
+#undef widget
 #include "playlistcollection.moc"
+
+// vim: set et sw=4:
