@@ -77,6 +77,8 @@ TracksImp::TracksImp( QWidget* parent, const char* name) :
 	QTimer *timer = new QTimer( this );
 	connect( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
 	timer->start( 1500, false ); // 1.5 seconds forever timer
+
+	wm_cd_init(WM_CDIN, (char *)qstrdup(QFile::encodeName(device)), NULL, NULL, NULL);
 }
 
 /**
@@ -96,6 +98,8 @@ TracksImp::~TracksImp() {
 
 	Prefs::setDevice(list);
 	Prefs::writeConfig();
+
+	wm_cd_destroy();
 }
 
 /**
@@ -136,9 +140,7 @@ bool TracksImp::hasCD(){
  * this routine.
  */
 void TracksImp::timerDone() {
-	wm_cd_init(WM_CDIN, (char *)qstrdup(QFile::encodeName(device)), NULL, NULL, NULL);
 	lookupDevice();
-	wm_cd_destroy();
 }
 
 void TracksImp::lookupDevice() {
@@ -169,7 +171,7 @@ void TracksImp::lookupDevice() {
 		return;
 	}
 
-	QString currentDistID = QString::number(cddb_discid(), 16);
+	QString currentDistID = QString::number(cddb_discid(), 16).rightJustify(8, '0');
 	if (currentDistID == cddbInfo.id) {
 		return;
 	}
@@ -240,6 +242,9 @@ void TracksImp::changeDevice(const QString &file ) {
 
 	device = file;
 
+	wm_cd_destroy();
+	wm_cd_init(WM_CDIN, (char *)qstrdup(QFile::encodeName(device)), NULL, NULL, NULL);
+
 	KApplication::setOverrideCursor(Qt::waitCursor);
 	timerDone();
 	KApplication::restoreOverrideCursor();
@@ -249,7 +254,7 @@ void TracksImp::changeDevice(const QString &file ) {
  * Helper function (toolbar button) for users.
  **/ 
 void TracksImp::performCDDB() {
-	if (hasCD()) {
+	if (!hasCD()) {
 		KMessageBox::sorry(this, i18n("Please insert a disk."),
 		       i18n("CDDB Failed"));
 		return;
@@ -286,6 +291,7 @@ void TracksImp::lookupCDDBDone(CDDB::Result result ) {
 	// TODO Why doesn't libcddb not return MultipleRecordFound?
 	//if( result == KCDDB::CDDB::MultipleRecordFound ) {
 	if( Prefs::promptIfIncompleteInfo() && cddb->lookupResponse().count() > 1 ) {
+		QString searchedCDId = cddbInfo.id;
 		CDInfoList cddb_info = cddb->lookupResponse();
 		CDInfoList::iterator it;
 		QStringList list;
@@ -321,6 +327,11 @@ void TracksImp::lookupCDDBDone(CDDB::Result result ) {
 			return;
 			// user pressed Cancel
 		}
+	// Check that the CD we looked up is the one now loaded.
+	// The user might have ejected the CD while we were in the
+	// KInputDialog event loop, and replaced it with another one.
+		if ( searchedCDId != cddbInfo.id )
+			return;
 	}
 
 	// Some sanity provisions to ensure that the number of records matches what
