@@ -125,6 +125,7 @@ KSCD::KSCD( QWidget *parent, const char *name )
   random_current      = random_list.begin();
 
   cddb = new KCDDB::Client();
+  connect(cddb, SIGNAL(finished(CDDB::Result)), this, SLOT(lookupCDDBDone(CDDB::Result)));
 
 #if defined(BUILD_CDDA)
   audio_systems_list
@@ -686,6 +687,7 @@ void KSCD::closeEvent(QCloseEvent *e)
         e->ignore();
         return;
     }
+    e->accept();
 }
 
 void KSCD::randomSelected()
@@ -1227,6 +1229,8 @@ void KSCD::CDDialogSelected()
 
         cddialog->setData(cddbInfo, trackStartFrames, playlist);
         connect(cddialog,SIGNAL(cddbQuery()),SLOT(lookupCDDB()));
+        connect(cddialog,SIGNAL(newCDInfoStored(KCDDB::CDInfo)),
+            SLOT(setCDInfo(KCDDB::CDInfo)));
         connect(cddialog,SIGNAL(finished()),SLOT(CDDialogDone()));
         connect(cddialog,SIGNAL(play(int)),SLOT(trackSelected(int)));
     }
@@ -1265,7 +1269,6 @@ void KSCD::lookupCDDB()
 
     cddb->config().reparse();
     cddb->setBlockingMode(false);
-    connect(cddb, SIGNAL(finished(CDDB::Result)), this, SLOT(lookupCDDBDone(CDDB::Result)));
 
     cddb->lookup(trackStartFrames);
 } // lookupCDDB
@@ -1323,7 +1326,18 @@ void KSCD::lookupCDDBDone(CDDB::Result result)
         // user pressed Cancel
       }
     }
+
+    setCDInfo(info);
   
+    // In case the cddb dialog is open, update it
+    if (cddialog)
+      cddialog->setData(cddbInfo, trackStartFrames, playlist);
+
+    led_off();
+} // lookupCDDBDone
+
+void KSCD::setCDInfo(KCDDB::CDInfo info)
+{
     // Some sanity provisions to ensure that the number of records matches what
     // the CD actually contains.
     while (info.trackInfoList.count() < cddbInfo.trackInfoList.count())
@@ -1336,20 +1350,14 @@ void KSCD::lookupCDDBDone(CDDB::Result result)
     }
     cddbInfo = info;
     populateSongList("");
-
-    // In case the cddb dialog is open, update it
-    if (cddialog)
-      cddialog->setData(cddbInfo, trackStartFrames, playlist);
-
-    led_off();
-} // lookupCDDBDone
+}
 
 void KSCD::lookupDevice(void)
 {
   // Initialise the album from the CD.
   cddbInfo.clear();
   trackStartFrames.clear();
-  cddbInfo.id = QString::number(cddb_discid(), 16);
+  cddbInfo.id = QString::number(cddb_discid(), 16).rightJustify(8,'0');
   unsigned numberOfTracks = wm_cd_getcountoftracks();
   cddbInfo.length = cd->trk[numberOfTracks].start - cd->trk[0].start;
   struct cdtext_info *info = wm_cd_get_cdtext();
@@ -1517,7 +1525,9 @@ QString KSCD::calculateDisplayedTime(int sec)
 
 QString KSCD::calculateDisplayedTime(int sec, int track)
 {
-    // should check if tracknumber is valid.
+	if( track == 0 )
+			return QString::null;
+	// should check if tracknumber is valid.
     static int mymin;
     static int mysec;
     int tmp;
