@@ -6,17 +6,30 @@
 
 class CModule;
 
+/**
+ * Noatun configuration dialog
+ **/
 class NoatunPreferences : public KDialogBase
 {
 Q_OBJECT
 friend class CModule;
 
 public:
+	/**
+	 * @internal
+	 **/
 	NoatunPreferences(QWidget *);
 
 public:
+	/**
+	 * Display noatun preferences dialog
+	 **/
 	virtual void show();
-	virtual void show(CModule *);
+	/**
+	 * Display noatun preferences dialog showing @p module
+	 * Useful if you want to display your own plugin configuration tab
+	 **/
+	virtual void show(CModule *module);
 
 protected:
 	virtual void slotOk();
@@ -33,6 +46,12 @@ private:
 	QPtrList<CModule> mModules;
 };
 
+/**
+ * @short Base class for a configuration sheet that is shown in preferences dialog
+ *
+ * Create your GUI in constructor, reimplement reopen() and save() and
+ * you're all set.
+ **/
 class CModule : public QWidget
 {
 Q_OBJECT
